/***************************************************************************
r6 (09/11/2016)
предназначена для реального полета
- занижен уровень земли - чтоб не было отрицательных значений 12 = 100м
// для теста - разность высот срабатывания при падении - уменьшена до 1м
- отключена проверка "запала"
// для теста - число подряд "понижений уменьшено до 3
// для теста - уменьшена пауза перед записью до 20 сек
- введен анализ начала полета
----------------------------------
фиксация замеров давления с периодичностью tPress (100 мс)
при старте анализ перемычки/кнопки на входе pinIN
если = 0, (кнопка нажата) то регистрация
если = 1, то вывод по UART
r5 - в EEPROM не пишется время, формат памяти: 
0			температура	до старта
2			давление /10	
4			напряжение	
6			запал	
8			температура	перед вкл запала
10			давление  /10	
12			напряжение	
14			запал	
16			напряжение	после вкл.запала
18			температура	в конце
20			давление   /10	
22			напряжение	
24			адрес вкл запала	
26			базовая (нулевая) высота - до взлета
28			отсчет высоты…	*10

  This is a library for the BMP280 humidity, temperature & pressure sensor

  Designed specifically to work with the Adafruit BMEP280 Breakout 
  ----> http://www.adafruit.com/products/2651

  These sensors use I2C or SPI to communicate, 2 or 4 pins are required 
  to interface.

  Adafruit invests time and resources providing this open source code,
  please support Adafruit andopen-source hardware by purchasing products
  from Adafruit!

  Written by Limor Fried & Kevin Townsend for Adafruit Industries.  
  BSD license, all text above must be included in any redistribution
 ***************************************************************************/

//#include <Wire.h>
#include <SPI.h>
#include <Adafruit_Sensor.h>
#include <Adafruit_BMP280.h>
#include <EEPROM.h>


//#define BMP_SCK 13
//#define BMP_MISO 12
//#define BMP_MOSI 11 
#define BMP_CS 10
#define led 9     //контрольный светодиод (к GND)
#define pinIN 8   //перемычка на GND
#define prd433 5  //на модулятор передатчика
#define ignit 7   //включение "запала" (затвор полевика, 1 - вкл)
#define ignitIN A2  //контроль "запала", 1 (>3V) - Ok
#define RAW_IN A5  //контроль питания 9V


//Adafruit_BMP280 bme; // I2C
Adafruit_BMP280 bme(BMP_CS); // hardware SPI
//Adafruit_BMP280 bme(BMP_CS, BMP_MOSI, BMP_MISO,  BMP_SCK);

#define N0 28   //начало области EEPROM для отсчетов высоты
#define Alt0  1025.25   //давление на высоте 0
#define MasMax 20       //размер массива в ОЗУ (по 2 байта) для записи отсчетов до старта
int addr = N0;  //пишем отсчеты не с начала EEPROM
int itmp = 0;
#define tPress 100   //период опроса датчика
#define maxAddr 1024 //объем EEPROM
#define LevIgnitMIN (1024 * 3 / 5)  //норма для контроля "запала" (выше 3V)
int maxAlt = 0;      //значение максимальной высоты
#define maxN 5  //3  //5       //число подряд идущих измерений высот меньших макс для опр.снижения
#define minD 60  //10   //100  //минимальное снижение от максимума для опр.снижения (на более чем 10м)
int N = 0;			 //подсчет числа повторов снижения высоты (до maxN)
int ignitON = 0;	 //признак "включен ли запал" (0 - не включен)
byte aLow;
byte aHigh;
unsigned int valAlt;
long voltage;
//-------------
int Nbuf = 0;          //указатель ОЗУ (до старта отсчеты пишем в ОЗУ)
byte Mas[MasMax*2];    //сам массив в ОЗУ для хранения отсчетов до взлета
unsigned int Hmax = 0;   //тут отслеживаем высоту при определении взлета 
unsigned int H0 = 0;     //тут высота на старте 
int Kpovt = 0;         //число повторов увеличения высоты при взлете
#define Hd 10 //50          //константа для определения взлета (5м)
#define Kmax 3 //5         //константа число подряд изм.высоты больше макс. для опр.взлета

//--------------- подпрограммы ---------------------------
//перезапись массива из ОЗУ в EEPROM после взлета
void MasToEE() {
  //запись в EEPROM штатного замера высоты
  valAlt = bme.readAltitude(Alt0) * 10; // -100м
//    Serial.print(valAlt);  //bme.readAltitude(1013.25)); // this should be adjusted to your local forcase
  EEPROM.write(addr, lowByte(valAlt));
  EEPROM.write(addr+1, highByte(valAlt));
  addr ++;  addr ++;   //к следующему адресу
  //если не весь массив из ОЗУ переписан в EEPROM - потихоньку переписываем
  if (Nbuf > 0) {
    //еще не весь массив перенесен в EEPROM
    EEPROM.write(Nbuf + N0, Mas[Nbuf]);   //одна 2х байтовая ячейка
    EEPROM.write(Nbuf + N0 + 1, Mas[Nbuf+1]);
    Nbuf --; Nbuf --;  //смещаем указатель массива
    EEPROM.write(Nbuf + N0, Mas[Nbuf]);   //еще одна ячейка
    EEPROM.write(Nbuf + N0 + 1, Mas[Nbuf+1]);
    Nbuf --; Nbuf --;  //смещаем указатель массива
  }
}

//запись отсчетов в массив в ОЗУ до старта
void AltToMas(unsigned int tAlt) {
  Mas[Nbuf] = lowByte(tAlt);
  Mas[Nbuf+1] = highByte(tAlt);
  Nbuf ++; Nbuf ++;
  if (Nbuf > MasMax * 2) {
    //весь массив заполнен, надо сдвигать
    for (Nbuf = 2; Nbuf <= MasMax*2; Nbuf ++) {
       Mas[Nbuf-1] = Mas[Nbuf];
    }
    Nbuf = (MasMax-1)*2; //ставим на последнюю ячейку
  }  
}
//--------------------------------------------------------

void setup() {
  // initialize the digital pin as an output.
  pinMode(led, OUTPUT);     
  pinMode(prd433, OUTPUT);     
  digitalWrite(prd433, LOW);   // выключить передатчик
  pinMode(ignit, OUTPUT);     
  digitalWrite(ignit, LOW);   // выключить "запал"
  pinMode(pinIN, INPUT); 
  digitalWrite(pinIN, HIGH);   // включим резистор к VCC
  
  if (digitalRead(pinIN) == HIGH) {
    //кнопка не нажата - читаем EEPROM
     Serial.begin(9600);
     Serial.println("Read EEPROM from MC & BMP280. Ver bmp280r6");
     for (addr=0; addr<maxAddr; addr = addr +2) {
 //       byte tLow  = EEPROM.read(addr);
 //       byte tHigh = EEPROM.read(addr+1);
        aLow  = EEPROM.read(addr);
        aHigh = EEPROM.read(addr+1);
  
  Serial.print(addr);
  Serial.print(";  ");
  switch (addr) {
  case 0: {
      Serial.print("T0;  ");
      break;
  }
  case 2: {
      Serial.print("P0;  ");
      break;
  }
  case 4: {
      Serial.print("V0;  ");
      break;
  }
  case 6: {
      Serial.print("Z0;  ");
      break;
  }
  case 8: {
      Serial.print("TM;  ");
      break;
  }
  case 10: {
      Serial.print("PM;  ");
      break;
  }
  case 12: {
      Serial.print("VM;  ");
      break;
  }
  case 14: {
      Serial.print("ZM;  ");
      break;
  }
  case 16: {
      Serial.print("VZ;  ");
      break;
  }
  case 18: {
      Serial.print("TE;  ");
      break;
  }
  case 20: {
      Serial.print("PE;  ");
      break;
  }
  case 22: {
      Serial.print("VE;  ");
      break;
  }
  case 24: {
      Serial.print("AM;  ");
      break;
  }
  case 26: {
      Serial.print("H0;  ");
      break;
  }
  default:  {  //отсчет высоты
      Serial.print("H;  ");
      break;
  }
  }
//  Serial.print(tLow+tHigh*256);  //время в мс
//  Serial.print(";  ");
  Serial.print(aLow+aHigh*256);  //высота в десятых долях метра
  Serial.print(";  ");
  Serial.println();
      
     }
   
     Serial.println("The End Read EEPROM");
     while(1);
  }  
/*  отключено
  int LevIgnitIN = analogRead(ignitIN);  //контроль "запала"
  if (LevIgnitIN < LevIgnitMIN)  { // должно быть больше 3V
  while(1) { 
    digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
    delay(250);                // wait for a 1/4 second
    digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW
    delay(250);                // wait for a second
    digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
    delay(250);                // wait for a 1/4 second
    digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW
    delay(250);                // wait for a second
     digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
    delay(250);                // wait for a 1/4 second
    digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW
    delay(750);                // wait for a second
   //частые "тройные" мигания если ошибка "запала" (обрыв)
  }
  }
  */
  
//  Serial.begin(9600);
//  Serial.println(F("BMP280 test"));
  
  if (!bme.begin()) {  
//    Serial.println(F("Could not find a valid BMP280 sensor, check wiring!"));
  while(1) { 
    digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
    delay(250);                // wait for a 1/4 second
    digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW
    delay(250);                // wait for a second
    //частые мигания если ошибка обнаружения BMP280
  }
//    while (1);
  }
  //пауза перед началом обнаружения старта - двойные вспышки
  for (itmp = 0; itmp < 3; itmp++) {  //30
     digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
	 analogWrite(prd433,127);   //вкл.модуляция перед. 1000Гц
	 delay(200);
     digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW
	 digitalWrite(prd433,LOW);   //выкл.модуляция перед. 
	 delay(200);
     digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
	 analogWrite(prd433,127);   //вкл.модуляция перед. 1000Гц
	 delay(200);
     digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW
	 digitalWrite(prd433,LOW);   //выкл.модуляция перед. 
	 delay(2000);
  }    
   
//=========== пишем параметры на старте ===========
//"оживим датчик"
  valAlt = bme.readTemperature(); //температура с BMP280    000
  long int valPress = bme.readPressure();  //давление в Паскалях (101000 Pa)
  valAlt = bme.readAltitude(1013.25) * 10; //analogRead(0) / 4;
  delay(100);
//снова измерим и запишем
  valAlt = (bme.readTemperature() +20)*100; //температура с BMP280    на 20 градусов выше и в 100 раз больше
  EEPROM.write(0, lowByte(valAlt));
  EEPROM.write(1, highByte(valAlt));
  valAlt = bme.readPressure()/10; //давление с BMP280 в 10 раз меньше
  EEPROM.write(2, lowByte(valAlt));
  EEPROM.write(3, highByte(valAlt));
  valAlt = analogRead(RAW_IN);    //питание устройства 9V    006
  EEPROM.write(4, lowByte(valAlt));
  EEPROM.write(5, highByte(valAlt));
  valAlt = analogRead(ignitIN);   //контроль питания "запала" (в норме должно быть как и 9V, при вкл - падение до 0) 
  EEPROM.write(6, lowByte(valAlt));
  EEPROM.write(7, highByte(valAlt));
  Hmax = bme.readAltitude(Alt0) * 10; //analogRead(0) / 4;
  H0 = Hmax;   //высота перед стартом
  EEPROM.write(26, lowByte(H0));
  EEPROM.write(77, highByte(H0));

  //пишем отсчеты в ОЗУ до обнаружения старта
     Serial.begin(9600);
     Serial.println("test upper. Ver bmp280r6");
 do {
    long int tTek = millis();
   valAlt = bme.readAltitude(Alt0) * 10; // -100м
   AltToMas(valAlt);  //пишем отсчет в ОЗУ
   Serial.print(valAlt);  //высота в десятых долях метра
   Serial.print(":");  //высота в десятых долях метра
     //контрольное мигание по три вспышки 1/8   2 сек
      int tTmp = highByte(tTek) & B00000111;
      if ((tTmp == 0) || (tTmp == 2) || (tTmp == 4))  {   
          digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
	  analogWrite(prd433,127);   //вкл.модуляция перед. 1000Гц
          }
      else {
          digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW
	  digitalWrite(prd433,LOW);   //выкл.модуляция перед. 
          } 
          
    if (valAlt >= Hmax) {
       Hmax = valAlt;
       Kpovt ++;      
     }
    else {
       Kpovt = 0;  //сброс числа повторений
     } 
    Serial.print(Kpovt);  //высота в десятых долях метра
    Serial.print("=");  //высота в десятых долях метра
    Serial.println(Hmax);  //высота в десятых долях метра
   while ((tTek + tPress) > millis()) {}  //пауза перед след.отсчетом
  } while(Hmax < (H0 + Hd) || Kpovt < Kmax);   //пока высота не увеличилась на Hd и число повторов роста высоты меньше Кмакс
  //летим...  идем в основной цикл регистрации
  Nbuf = MasMax*2;           //подготовка к перезаписи ОЗУ в EEPROM
  addr = MasMax*2 + 2 + N0;  //адрес EEPROM для фиксации высоты
}

void loop() {
  long int tTek = millis();
  
  //контрольное мигание 1/8   2 сек
  int tTmp = highByte(tTek) & B00000111;
  if (tTmp == 0)  {   
    digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
	analogWrite(prd433,127);   //вкл.модуляция перед. 1000Гц
  }
  else {
    digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW
	digitalWrite(prd433,LOW);   //выкл.модуляция перед. 
  }    
//    Serial.print(F("Temperature = "));
//    Serial.print(bme.readTemperature());
//    Serial.println(" *C");
 
//Давление    
//  long int valPress = bme.readPressure();  //давление в Паскалях (101000 Pa)
//    Serial.print(F("Pressure = "));
//    Serial.print(bme.readPressure());
//    Serial.println(" Pa");

//Высота
//    Serial.print(F("Approx altitude = "));
//  unsigned int valAlt = bme.readAltitude(1013.25) * 10; //analogRead(0) / 4;
  valAlt = bme.readAltitude(Alt0) * 10; // -100м
    
//    Serial.print(valAlt);  //bme.readAltitude(1013.25)); // this should be adjusted to your local forcase
//    Serial.println(" m");
    
//    Serial.println();
  
  // write the value to the appropriate byte of the EEPROM.
  // these values will remain there when the board is
  // turned off.
  
  //надо записать:
  //  два младших байта из tTek         2
  //  два байта из valAlt               2
  //(может быть и давление)
  //  четыре байта из valPress          4
//  EEPROM.write(addr, lowByte(tTek));
//  EEPROM.write(addr+1, highByte(tTek));
  EEPROM.write(addr, lowByte(valAlt));
  EEPROM.write(addr+1, highByte(valAlt));
  
  //определяем момент включения "запала системы спасения"
  if (ignitON == 0) {  //только если запал еще не включали 
  if (valAlt > maxAlt) {  //"копим" максимум - идет "взлет"
      maxAlt = valAlt;
	  N = 0;
	  }
  else {  //возможно, пошло падение
      N ++;  //считаем число повторов "понижения"
	  if (N > maxN) {  //число подряд "понижений" превысило порог - можно сказать - "падаем"
	      //проверим абсолютное значение "падения" к максимуму
	      if ((maxAlt - valAlt) > minD) { //тут пора включать "запал"
		      //запишем данные точки
  valAlt = (bme.readTemperature() +20)*100; //температура с BMP280    на 20 градусов выше и в 100 раз больше
  EEPROM.write(8, lowByte(valAlt));
  EEPROM.write(9, highByte(valAlt));
  valAlt = bme.readPressure()/10; //давление с BMP280 в 10 раз меньше
  EEPROM.write(10, lowByte(valAlt));
  EEPROM.write(11, highByte(valAlt));
  valAlt = analogRead(RAW_IN);    //питание устройства 9V    006
  EEPROM.write(12, lowByte(valAlt));
  EEPROM.write(13, highByte(valAlt));
  valAlt = analogRead(ignitIN);   //контроль питания "запала" (в норме должно быть как и 9V, при вкл - падение до 0) 
  EEPROM.write(14, lowByte(valAlt));
  EEPROM.write(15, highByte(valAlt));
			  //запишем в EEPROM момент включения "запала" (addr-1)
				EEPROM.write(24, lowByte(addr));
				EEPROM.write(25, highByte(addr));
			  //включим "запал"
				digitalWrite(ignit, HIGH);
				ignitON = 1; //признак "Запал включен"
  delay(10); //подождем реакции
  valAlt = analogRead(RAW_IN);    //питание устройства 9V    после запала
  EEPROM.write(16, lowByte(valAlt));
  EEPROM.write(17, highByte(valAlt));
		     }
	     }
	  }
  }
  // advance to the next address.  there are 512 bytes in 
  // the EEPROM, so go back to 0 when we hit 512.
  addr = addr + 2;
  if (addr >= maxAddr)  {
 //   addr = 0;  //циклическая перезапись (!?)
 // запись завершена - фиксируем конечные данные
  valAlt = (bme.readTemperature() +20)*100; //температура с BMP280    на 20 градусов выше и в 100 раз больше
  EEPROM.write(18, lowByte(valAlt));
  EEPROM.write(19, highByte(valAlt));
  valAlt = bme.readPressure()/10; //давление с BMP280 в 10 раз меньше
  EEPROM.write(20, lowByte(valAlt));
  EEPROM.write(21, highByte(valAlt));
  valAlt = analogRead(RAW_IN);    //питание устройства 9V    006
  EEPROM.write(22, lowByte(valAlt));
  EEPROM.write(23, highByte(valAlt));
//память заполнена - светодиод включаем непрерывно
     digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
     while (1) {  //непрерывно передаем "Ж" (...-)
		analogWrite(prd433,127);   //вкл.модуляция перед. 1000Гц
		delay(200);
		digitalWrite(prd433,LOW);   //выкл.модуляция перед. 
		delay(200);
		analogWrite(prd433,127);   //вкл.модуляция перед. 1000Гц
		delay(200);
		digitalWrite(prd433,LOW);   //выкл.модуляция перед. 
		delay(200);
		analogWrite(prd433,127);   //вкл.модуляция перед. 1000Гц
		delay(200);
		digitalWrite(prd433,LOW);   //выкл.модуляция перед. 
		delay(200);
		analogWrite(prd433,127);   //вкл.модуляция перед. 1000Гц
		delay(600);
		digitalWrite(prd433,LOW);   //выкл.модуляция перед. 
		delay(800);
	 } ;
  }
    
  while ((tTek + tPress) > millis()) {} 
//    delay(100);
}
